# Makefile to build the composite D runtime library for Win32
# Designed to work with DigitalMars make
# Targets:
#	make
#		Same as make all
#	make lib
#		Build the runtime library
#   make doc
#       Generate documentation
#	make clean
#		Delete unneeded files created by build process

LIB_BASE=tango-base-dmd
LIB_BUILD=
LIB_TARGET=$(LIB_BASE)$(LIB_BUILD).lib
LIB_MASK=$(LIB_BASE)*.lib

DIR_CC=common\tango
DIR_RT=compiler\dmd
DIR_GC=gc\basic

LIB_CC=$(DIR_CC)\tango-cc-tango$(LIB_BUILD).lib
LIB_RT=$(DIR_RT)\tango-rt-dmd$(LIB_BUILD).lib
LIB_GC=$(DIR_GC)\tango-gc-basic$(LIB_BUILD).lib

CP=xcopy /y
RM=del /f
MD=mkdir

CC=dmc
LC=lib
DC=dmd

ADD_CFLAGS=
ADD_DFLAGS=
CFLAGS_RELEASE=-mn -6 -r $(ADD_CFLAGS)
CFLAGS_DEBUG=-g -mn -6 -r $(ADD_CFLAGS)
DFLAGS_RELEASE=-release -O -inline -w -nofloat -I. -I../shared -I../../.. $(ADD_DFLAGS)
DFLAGS_DEBUG=-g -w -nofloat  -I. -I../shared -I../../.. $(ADD_DFLAGS)
TFLAGS_RELEASE=-O -inline -w  -nofloat $(ADD_DFLAGS)
TFLAGS_DEBUG=-g -w -nofloat $(ADD_DFLAGS)

targets : lib-release doc
all     : lib-release lib-debug doc

######################################################

ALL_OBJS=

######################################################

ALL_DOCS=

######################################################
lib: lib-release

lib-release:
	make -fdmd-win32.mak clean LIB_BUILD=""
	make -fdmd-win32.mak release-comp LIB_BUILD=""

lib-debug:
	make -fdmd-win32.mak clean LIB_BUILD=""
	make -fdmd-win32.mak debug-comp LIB_BUILD="-d"
	
release-comp : $(ALL_OBJS)
	cd $(DIR_CC)
	make -fwin32.mak lib DC=$(DC) DFLAGS="$(DFLAGS_RELEASE)" CFLAGS="$(CFLAGS_RELEASE)" \
		TFLAGS="$(TFLAGS_RELEASE)" LIB_BUILD="$(LIB_BUILD)"
	cd ..\..
	cd $(DIR_RT)
	make -fwin32.mak lib DC=$(DC) DFLAGS="$(DFLAGS_RELEASE)" CFLAGS="$(CFLAGS_RELEASE)" \
		TFLAGS="$(TFLAGS_RELEASE)" LIB_BUILD="$(LIB_BUILD)"
	cd ..\..
	cd $(DIR_GC)
	make -fwin32.mak lib DC=$(DC) DFLAGS="$(DFLAGS_RELEASE)" CFLAGS="$(CFLAGS_RELEASE)" \
		TFLAGS="$(TFLAGS_RELEASE)" LIB_BUILD="$(LIB_BUILD)"
	cd ..\..
	$(RM) $(LIB_TARGET)
	$(LC) -c -n $(LIB_TARGET) $(LIB_CC) $(LIB_RT) $(LIB_GC)

debug-comp : $(ALL_OBJS)
	cd $(DIR_CC)
	make -fwin32.mak lib DC=$(DC) DFLAGS="$(DFLAGS_DEBUG)" CFLAGS="$(CFLAGS_DEBUG)" \
		TFLAGS="$(TFLAGS_DEBUG)" LIB_BUILD="$(LIB_BUILD)"
	cd ..\..
	cd $(DIR_RT)
	make -fwin32.mak lib DC=$(DC) DFLAGS="$(DFLAGS_DEBUG)" CFLAGS="$(CFLAGS_DEBUG)" \
		TFLAGS="$(TFLAGS_DEBUG)" LIB_BUILD="$(LIB_BUILD)"
	cd ..\..
	cd $(DIR_GC)
	make -fwin32.mak lib DC=$(DC) DFLAGS="$(DFLAGS_DEBUG)" CFLAGS="$(CFLAGS_DEBUG)" \
		TFLAGS="$(TFLAGS_DEBUG)" LIB_BUILD="$(LIB_BUILD)"
	cd ..\..
	$(RM) $(LIB_TARGET)
	$(LC) -c -n $(LIB_TARGET) $(LIB_CC) $(LIB_RT) $(LIB_GC)

doc : $(ALL_DOCS)
	cd $(DIR_CC)
	make -fwin32.mak doc
	cd ..\..
	cd $(DIR_RT)
	make -fwin32.mak doc
	cd ..\..
	cd $(DIR_GC)
	make -fwin32.mak doc
	cd ..\..

######################################################

clean :
	$(RM) /s *.di
	$(RM) $(ALL_OBJS)
	$(RM) $(ALL_DOCS)
	cd $(DIR_CC)
	make -fwin32.mak clean LIB_BUILD="$(LIB_BUILD)"
	cd ..\..
	cd $(DIR_RT)
	make -fwin32.mak clean LIB_BUILD="$(LIB_BUILD)"
	cd ..\..
	cd $(DIR_GC)
	make -fwin32.mak clean LIB_BUILD="$(LIB_BUILD)"
	cd ..\..

clean-all: clean
	cd $(DIR_CC)
	make -fwin32.mak clean-all
	cd ..\..
	cd $(DIR_RT)
	make -fwin32.mak clean-all
	cd ..\..
	cd $(DIR_GC)
	make -fwin32.mak clean-all
	cd ..\..
	$(RM) $(LIB_MASK)
	
install :
	cd $(DIR_CC)
	make -fwin32.mak install
	cd ..\..
	cd $(DIR_RT)
	make -fwin32.mak install
	cd ..\..
	cd $(DIR_GC)
	make -fwin32.mak install
	cd ..\..
#	$(CP) $(LIB_MASK) $(LIB_DEST)\.
